/*
 * Copyright: STEM Loyola
 * Date     : August 2019
 * ID       : 19.08-B2
 * Level    : 2 (Intermediate)
 *
 * Task     : In 2014, NECTA released the form four (CSEE) results in GPA and
 *            class format (Figure 2). This came after they had introduced "B+"
 *            and "E" grades during the 2013 results. Your task is to convert
 *            the results to the points and divisions format (Figure 3) that we
 *            are all familiar with and count students in each division. A text
 *            file "2014-CSEE-S0800.txt" contains the results in GPA format as
 *            shown in Figure 1. No one got division zero but we have added one
 *            division-zero results to ensure you write a complete program.
 *
 *      Note: (1) Points and divisions are calculated from the 7 best grades as
 *                shown in Figure 4 and Figure 5.
 *            (2) In order to get Division IV, a student must have at least
 *                2 Ds or 1 C. This means, for example, that 6 Es and 1 D
 *                (i.e. 41 pts) will still be Division 0.
 *
 * Solved By: STEM Loyola Team
 *     Email: stemloyola@gmail.com
 *     Form : N/A
 *    Stream: N/A
*/

import java.io.File;
import java.util.Scanner;
import java.util.HashMap;
import java.util.Collections;
import java.util.ArrayList;

class STEMChallenge05Solved {
    
    static final String FILE_NAME     = "2014-CSEE-S0800.txt";
    static final int    MIN_SUBJECTS  = 7;
    
    public static void main(String args[]) {
        // Open the results file and ensure it was opened successfully
        Scanner resultsScanner = null;

        try {
            resultsScanner = new Scanner(new File(FILE_NAME));

        } catch (Exception e) {
            System.err.format("Error: could not open the file \"%s\"\n", FILE_NAME);
            System.err.format("Quitting...\n");

            System.exit(1);    // Terminate the program
        }

        int totalDivisions[] = { 0, 0, 0, 0, 0 };  // Tracks students in each division

        HashMap<String, Integer> gradeToPoints = new HashMap<String, Integer>();
        gradeToPoints.put("A",  1);
        gradeToPoints.put("B+", 2);
        gradeToPoints.put("B",  3);
        gradeToPoints.put("C",  4);
        gradeToPoints.put("D",  5);
        gradeToPoints.put("E",  6);
        gradeToPoints.put("F",  7);

        // Process the results file
        String line;

        while ( resultsScanner.hasNextLine() ) {
            line = resultsScanner.nextLine();

            String[] items = line.split(" ", 3);
            String studID  = items[0];
            String gender  = items[1];
            String subjects = items[2];

            // Extract all subjects and grades for the current student
            ArrayList<Integer> pointsList = new ArrayList<Integer>();
            String[] subjectsList = subjects.split("'");
            for ( int i = 1; i < subjectsList.length; i += 2 ) {
                pointsList.add(gradeToPoints.get(subjectsList[i]));
            }

            // Compute and sum up the student's best seven points
            Collections.sort(pointsList);

            int points = 0;
            for ( int i = 0; i < MIN_SUBJECTS; i++ ) {
                points += pointsList.get(i);
            }

            // Compute the student's division.
            String division = "";

            if ( points <= 17 ) {
                division = "I";
                totalDivisions[1]++;
            } else if ( points <= 24 ) {
                division = "II";
                totalDivisions[2]++;
            } else if ( points <= 31 ) {
                division = "III";
                totalDivisions[3]++;
            } else if ( points <= 46 ) {
                // At least one C or two D's are needed to get division 4 even
                // if a student scores the right points
                if ( pointsList.get(0) <= 4 || pointsList.get(0) + pointsList.get(1) <= 10 ) {
                    division = "IV";
                    totalDivisions[4]++;
                } else {
                    division = "0";
                    totalDivisions[0]++;
                }
            } else {
                division = "0";
                totalDivisions[0]++;
            }

            System.out.printf("%s\t%s\t%d\t%s\n", studID, gender, points, division);
        }

        // Display the overall results summary
        System.out.printf("\nResults summary:\n\t");
        System.out.printf("Division I:    \t%d\n\t", totalDivisions[1]);
        System.out.printf("Division II:   \t%d\n\t", totalDivisions[2]);
        System.out.printf("Division III:  \t%d\n\t", totalDivisions[3]);
        System.out.printf("Division IV:   \t%d\n\t", totalDivisions[4]);
        System.out.printf("Division 0 (*):\t%d\n\t", totalDivisions[0]);

        // Close open resources
        resultsScanner.close();
    }
}
